# -*- coding: utf-8 -*-
#Calculate and plot the the separation (rho) and position angle (theta) of double stars given the double stars RA and Dec. 
#
#Plot and label the stars positions.
#Plotting a sphere#http://en.wikipedia.org/wiki/Map_projection
#The plotting uses plate carrée projection or geographic projection which is an equirectangular projection.
#The plotted RA and Dec lines are parallel and are the same distance at the celestrial equator.
#This projection maps x to be the value of the RA and y to be the value of the Dec.
#
#If multiple measurements calculate the mean and standard deviation for rho and theta.
#If multiple measurements calculate the mean and standard deviation for the star positions RA and Dec.
#Then calculate the the seperations (rho) and postion angle (theta) of mean RA and Dec.
#
#Used with comma seperate variable (.csv) file or plain text file (.txt) 
#'DoubleStarsData.txt'  with multiple rows with the following format without the Python comment '#' in the first character position.
#KUI 82, 5-30-2013, 9:46:46, Image, 5970, Primary RA, 17, 29, 19.86, Dec, 29, 23, 27.0, Secondary RA, 17, 29, 17.11, Dec, 29, 24, 1.4, P Label XY offset, 0,  1, S Label XY offset, 0, 1, Plot Size, 7, Circles, 6, 1, Plate Solve Software Used
#KUI 82, 5-30-2013, 9:51:33, Image, 5971, Primary RA, 17, 29, 19.84, Dec, 29, 23, 28.8, Secondary RA, 17, 29, 17.03, Dec, 29, 24, 3.0, P Label XY offset, 0,  1, S Label XY offset, 0, 1, Plot Size, 7, Circles, 6, 1, Plate Solve Software Used
#KUI 82, 5-30-2013, 9:51:43, Image, 5972, Primary RA, 17, 29, 19.76, Dec, 29, 23, 28.2, Secondary RA, 17, 29, 16.97, Dec, 29, 24, 2.2, P Label XY offset, 0,  1, S Label XY offset, 0, 1, Plot Size, 7, Circles, 6, 1, Plate Solve Software Used
#KUI 82, 5-30-2013, 9:46:46, Image, 5973, Primary RA, 17, 29, 19.92, Dec, 29, 23, 27.9, Secondary RA, 17, 29, 17.17, Dec, 29, 24, 2.4, P Label XY offset, 0,  1, S Label XY offset, 0, 1, Plot Size, 7, Circles, 6, 1, Plate Solve Software Used
#KUI 82, 5-30-2013, 9:55:31, Image, 5974, Primary RA, 17, 29, 19.63, Dec, 29, 23, 27.3, Secondary RA, 17, 29, 16.81, Dec, 29, 24, 2.6, P Label XY offset, 0,  1, S Label XY offset, 0, 1, Plot Size, 7, Circles, 6, 1, Plate Solve Software Used
#
#DoubleStarsData.txt format:
#KUI 82,            :Double star name
#5-30-2013,         :Date of observation
#9:46:46,           :Time of observation
#Image,             :Image number label field
#5970,              :Image number, needs to be a valid number
#Primary RA,        :Start of the primary star RA location field
#17,                :Primary star RA hours
#29,                :Primary star RA minutes
#19.86,             :Primary star RA seconds
#Primary Dec,       :Start of the primary star Dec location field
#29,                :Primary star Dec hours
#23,                :Primary star Dec minutes
#27.0,              :Primary star Dec seconds
#Secondary RA,      :Start of the secondary star RA location field
#17,                :Secondary star RA hours
#29,                :Secondary star RA minutes
#17.11,             :Secondary star RA seconds
#Secondary Dec,     :Start of the secondary star Dec location field
#29,                :Secondary star Dec hours
#24,                :Secondary star Dec minutes
#1.4,               :Secondary star Dec seconds
#P Label XY offset, :Start of the primary star image label XY offset field, if both X & X are 0 no label is plotted
#0,                 :Primary star image label X offset in 0.1 arcseconds 
#1,                 :Primary star image label y offset in 0.1 arcseconds 
#S Label XY offset, :Start of the secondary star image label XY offset field, if both X & X are 0 no label is plotted 
#0,                 :Secondary star image label X offset in 0.1 arcseconds 
#1,                 :Secondary star image label Y offset in 0.1 arcseconds 
#Plot Size,         :Start of the plot window XY size field
#7,                 :Plot window XY size in arcseconds 
#Circles,           :Start of the plot circles centered on average position field, 
#6,                 :Number of circles to plot, if 0 or less no circles are plotted
#1,                 :Circle radius in arcseconds, if 0 or less no circles are plotted
#Comment,           :Any text, I use for showing which Plate Solve Software used for RA and Dec data
#
#Python script developed with Enthought Canopy Express (Free) V1.1.0.1371 (64bit) for OS X
#https://www.enthought.com/products/canopy/compare-subscriptions/
#
#Apple MacBook Pro
#http://www.stargazing.net/david/equip/MacBookPro.html
#
#Script based on matplotlib Examples
#http://matplotlib.org/examples/index.html
#
#David Haworth
#http://www.stargazing.net/david/

import math
import matplotlib.pyplot as plt
import numpy as np
from pylab import *
import os
import sys
import datetime
from time import *

auto_aspect_ratio = True #True or False, False is 1 to 1 plot aspect ratio
plot_window_x_size = 9
plot_window_y_size = 8

def get_date_time():
    datetime = strftime("%Y-%m-%d %H:%M:%S").replace(' ', '_')
    datetime = datetime.replace(':', '')
    datetime = datetime.replace('-', '')
    return datetime

def hms_str(hours, minutes, seconds):
    return (str(int(hours))+'h '+ \
    str(int(minutes))+"' "+ \
    str(seconds)+'"')

def degrees_str(hours, minutes, seconds):
    return (str(int(hours))+' degrees'+' '+ \
    str(int(minutes))+"' "+ \
    str(seconds)+'"')

def deg_decimal_RA(hours, minutes, seconds):
    return (hours*360/24.0+minutes*360/(24*60.0)+seconds*360/(24*60.0*60))

def deg_decimal_Dec(degrees, minutes, seconds):
    if degrees < 0:
        if minutes > 0:
            minutes = -minutes
        if seconds > 0:
            seconds = -seconds     
    return (degrees+minutes*(1/60.0)+seconds*(1/3600.0))

def decimal_azimuth(degrees, minutes, seconds):
    return (degrees+minutes/60.0+seconds/3600.0) 
    
def deg_string(deg):
    return '%.5f degrees' %deg    
    
def RA_HMS(decimal_degrees):
    hours =  int(decimal_degrees/(360/24.0))  
    min_rem =  decimal_degrees - float(hours)*(260/24.0)
    minutes =  int(min_rem/(1/60.0))
    sec_rem =  min_rem - float(minutes)*(1/60.0)
    seconds = sec_rem/(1/3600.0)
    return hours, minutes, seconds

def rho_theta(ra1f, dec1f, ra2f, dec2f):
    ra1 = math.radians(ra1f)
    ra2 = math.radians(ra2f)

    dec1 = math.radians(dec1f)
    dec2 = math.radians(dec2f)
    
    rho = math.acos(math.cos((ra2-ra1)*math.cos(dec1))*math.cos(dec2-dec1))
    theta = (math.pi/2.0)-math.atan2(math.sin(dec2-dec1), math.cos(dec2-dec1)*math.sin((ra2-ra1)*math.cos(dec1)))

    dec = dec2-dec1
    ra = ra2-ra1
    if (ra < 0 and dec > 0) : theta = theta+math.pi*2
    return math.degrees(rho)*3600, math.degrees(theta)    

def circle_arcsec(num_arcsec):
    t = arange(0.0, 361, 1)
    a=num_arcsec*.5/3600.0
    s = a*sin(radians(t))
    c = a*cos(radians(t))
    return c, s

def circle_arcsec_theta(num_arcsec, theta):
    t = arange(90-theta, 90+1, 1)
    a=num_arcsec*.5/3600.0
    s = a*sin(radians(t))
    c = a*cos(radians(t))
    return c, s

def stretch_RA(dec): 
    stretch_factor = math.cos(math.radians(0))/math.cos(math.radians(dec))
    return stretch_factor

def plot_circles(num_of_circles, size, center_ra, center_dec):
    if num_of_circles > 0:
        if size > 0:
            for index in range(1,num_of_circles+1):
                xc, yc = circle_arcsec(index*size*2)
                yc+= center_dec
                for index2 in range(len(xc)):
                    xc[index2] = center_ra+xc[index2]*stretch_RA(yc[index2]) 
                plt.plot(xc, yc, color = '.5')
                plt.annotate(('%.3f arcsec radius' %(index*size)), xy=(center_ra, center_dec-(index*size*2*.5/3600.0)),  xycoords='data',
                    horizontalalignment='center', verticalalignment='top', color = '.5')  

def plot_rho_circle(size, center_ra, center_dec):
    xc, yc = circle_arcsec(size*2)
    yc+= center_dec
    for index2 in range(len(xc)):
        xc[index2] = center_ra+xc[index2]*stretch_RA(yc[index2]) 
    plt.plot(xc, yc, color = 'brown')
    plt.annotate(('Rho = %.3f arcseconds' %size), xy=(center_ra, center_dec-(size*2*.5/3600.0)),  xycoords='data',
        horizontalalignment='center', verticalalignment='top', color = 'brown')  

def plot_theta(center_ra, center_dec, rho, theta):
    xc, yc = circle_arcsec_theta(rho, theta)
    yc+= center_dec     
    for index2 in range(len(xc)):
        xc[index2] = center_ra+xc[index2]*stretch_RA(yc[index2]) 
    plt.plot(xc, yc, '--', color = 'g')
    
          
def plotNSEWlabels():    
    plt.annotate('West', xy=(.95, .5),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=12)                                               

    plt.annotate('East', xy=(.02, .5),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=12)                                                                                                                                             

    plt.annotate('North', xy=(.48, .98),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=12)                                                                                                                                                                                                                                                                                                                                                                                                                                   
                              
    plt.annotate('South', xy=(.48, .03),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=12)                                                                                                                                                                                                                                                                                                                                                                                                                                   
    
def plotAverageLabels(ra_p, dec_p, ra_s, dec_s):
    plt.annotate('Primary Average', xy=(ra_p, dec_p),  xycoords='data',
        xytext=(0.7, 0.4), textcoords='axes fraction',
        arrowprops=dict(arrowstyle="->"),
        horizontalalignment='right', verticalalignment='top') 

    plt.annotate('Secondary Average', xy=(ra_s, dec_s),  xycoords='data',
        xytext=(0.7, 0.4), textcoords='axes fraction',
        arrowprops=dict(arrowstyle="->"),
        horizontalalignment='right', verticalalignment='top')        

def plotFileNames(script, data):
    plt.annotate(script, xy=(.01, .06),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)
    
    plt.annotate(data, xy=(.01, .03),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)

def plotImageNum(image_num):
    if image_num != 0:
        plt.annotate(('Image %d' %image_num), xy=(.01, .09),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)
    
def plotPlotSize(x_size, y_size):
    plt.annotate(('Equirectangular map projection'), xy=(.6, .99),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)

    plt.annotate(('Plot size: %.2f by %.2f arcseconds' %(x_size, y_size)), xy=(.6, .96),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)
          
def plotComment(comment_str):
    plt.annotate(comment_str, xy=(.6, .03),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)

def get_script_name():
    python_file_name = os.path.basename(__file__)                          
    return python_file_name

def get_data_file_name():
    found_file=0
    while not found_file:
        data_file_name=raw_input('Enter file name or "Enter" key for directory listing > ')
        found_file=os.path.exists(data_file_name)
        if found_file == False:
            print 'Warning: file "'+data_file_name+'" not found in '+os.getcwd()+' directory!'
            print os.getcwd()+' directory listing:'
            for index in range(len(os.listdir(os.getcwd()))):
                print os.listdir(os.getcwd())[index]
    return data_file_name                

def get_row_count(data_file_name):
    image_num, p_ra_h=np.loadtxt(data_file_name, delimiter=',', usecols=(4,6), unpack=True)
    return image_num.size

def plotThetaRho(theta, rho):
    plt.annotate(('Theta = %.3f degrees' %theta), xy=(.01, .99),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)
    
    plt.annotate(('Rho =  %.3f arcseconds' %rho), xy=(.01, .96),
        xycoords='figure fraction',
        horizontalalignment='left', verticalalignment='top',
        fontsize=11)

def plot_double_star(python_file_name, data_file_name, txtfile, ds_name_str, comment, ra_p_s, dec_p_s, ra_s_s, dec_s_s, rhos, thetas):
    arcsecond = 1/3600.0                         
    offset = 0.1/3600.0

    figure(3)
    rcParams['figure.figsize'] = plot_window_x_size, plot_window_y_size
    if auto_aspect_ratio == True:
        axes().set_aspect(str(stretch_RA(dec_p_s)))
    else:
        axes().set_aspect('equal')    
    plt.title('Double Star '+ds_name_str+' AB,C', fontsize=16, fontweight='bold')
    plt.xlabel('RA in Degrees')
    plt.ylabel('Dec in Degrees')
          
    plot_theta(ra_p_s, dec_p_s, rhos, thetas)
    plt.annotate(('Theta = %.3f degrees' %thetas), 
        xy=(ra_p_s, dec_p_s+rhos/(2*3600.)),  
        xycoords='data', color = 'g',
        horizontalalignment='left', verticalalignment='center')     

    plt.plot([ra_p_s,ra_p_s ], [dec_p_s,dec_p_s+rhos/3600.], '--', color = 'g')
    plt.annotate('N', 
        xy=(ra_p_s, dec_p_s+rhos/3600.),  
        xycoords='data', color = 'g',
        horizontalalignment='center', verticalalignment='bottom') 
    plt.plot([ra_p_s,ra_s_s ], [dec_p_s,dec_s_s],  marker='*', markersize=18, color = 'blue')

    plt.annotate('Primary', 
        xy=(ra_p_s, dec_p_s+rhos*.07*1/3600.),  
        xycoords='data',
        horizontalalignment='center', verticalalignment='center')
   
    plt.annotate('Secondary', 
            xy=(ra_s_s, dec_s_s+rhos*.07*1/3600.),
            xycoords='data',
            horizontalalignment='center', verticalalignment='center')  

    plot_rho_circle(rhos, ra_p_s, dec_p_s)

    if auto_aspect_ratio == True:
        plt.xlim(ra_p_s+arcsecond*rhos*1.15*stretch_RA(dec_p_s), ra_p_s-arcsecond*rhos*1.15*stretch_RA(dec_p_s))
        plt.ylim(dec_p_s-arcsecond*rhos*1.15, dec_p_s+arcsecond*rhos*1.15)
        textData = ('\nDouble star AB plot size is %.2f by %.2f arcseconds.' %(2*rhos*1.15*stretch_RA(dec_p_s),2*rhos*1.15))
        print textData
        txtfile.write(textData+'\n')
        plotPlotSize(2*rhos*1.15*stretch_RA(dec_p_s),2*rhos*1.15)   
    else:
        plt.xlim(ra_p_s+arcsecond*rhos*1.15, ra_p_s-arcsecond*rhos*1.15)
        plt.ylim(dec_p_s-arcsecond*rhos*1.15, dec_p_s+arcsecond*rhos*1.15)
        textData = ('\nDouble star AB plot size is %.2f by %.2f arcseconds.' %(2*rhos*1.15,2*rhos*1.15))
        print textData
        txtfile.write(textData+'\n')
        plotPlotSize(2*rhos*1.15,2*rhos*1.15)                                                        

    plotFileNames(python_file_name, data_file_name)
    plotComment(comment)
    plotThetaRho(thetas, rhos)
    plotNSEWlabels()
    plt.grid(True) 
    file_name_date_time = get_date_time()
    savefig(ds_name_str+'_ABC_'+file_name_date_time+'.png')                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
                                                                
                                                                
def analyze_one_line(python_file_name, data_file_name):
    arcsecond = 1/3600.0                         
    offset = 0.1/3600.0

    image_num, p_ra_h, p_ra_m, p_ra_s,  p_dec_h, p_dec_m, p_dec_s, \
    s_ra_h, s_ra_m, s_ra_s, s_dec_h, s_dec_m, \
    s_dec_s, xlabelp, ylabelp, \
    xlabels, ylabels, plot_size, num_circles, circle_size \
    =np.loadtxt(data_file_name, delimiter=',', usecols=(4, 6,7,8, 10,11,12, 14,15,16, 18,19,20, 22,23, 25,26, 28,30,31), unpack=True)

    if image_num == 0:
        image_num_str = ''
    else:
        image_num_str = str(image_num)
    rows = p_ra_h.size
    
    thetas = zeros(rows)
    rhos = zeros(rows)
    ra_p_s = zeros(rows)
    dec_p_s = zeros(rows)
    ra_s_s = zeros(rows)
    dec_s_s = zeros(rows)

    data_file = open(data_file_name, 'r')
    first_data_row = data_file.readline()
    data_file.close()

    comma = first_data_row.find(',')
    ds_name_str =  first_data_row[0:comma]
    last_comma = first_data_row.rfind(',')
    comment = first_data_row[last_comma+1:]
    comment = comment.lstrip(' ')
    if ord(comment[len(comment)-1:len(comment)])<32:
        comment = comment = comment[0:len(comment)-1]
    if ord(comment[len(comment)-1:len(comment)])<32:
        comment = comment = comment[0:len(comment)-1]

    file_name_date_time = get_date_time()
    txtfile = open(ds_name_str+'_'+file_name_date_time+'_DataOut.txt', 'w')
    
    now = datetime.datetime.now()
    textData = now.strftime('Date: %Y-%m-%d time: %H:%M') 
    txtfile.write(textData+'\n')
    print textData
    
    textData = 'Python script file name: '+python_file_name+'\n'
    txtfile.write(textData+'\n')
    print textData

    textData = 'Data file: '+data_file_name+' has %.d row' %rows
    txtfile.write(textData+'\n')
    print textData
    textData = 'Data file contains:\n'
    txtfile.write(textData+'\n')
    print textData        
                        
    data_file = open(data_file_name, 'r')
    for index in range(rows):
        data_row = data_file.readline()
        textData = ('Row %d:' %index)+data_row.rstrip('\n')
        txtfile.write(textData+'\n')
        print textData
    data_file.close()    

    textData = '\nDouble star name:'+ds_name_str     
    print textData
    txtfile.write(textData+'\n')    
    textData = 'Comment: '+comment  
    print textData
    txtfile.write(textData+'\n')

    textData = ('\nImage %.1f analysis:' %image_num)
    print textData
    txtfile.write(textData+'\n')
    
    ra_p_s = deg_decimal_RA(p_ra_h, p_ra_m, p_ra_s)
    textData = '   Primary RA = '+hms_str(p_ra_h, p_ra_m, p_ra_s) 
    print textData
    txtfile.write(textData+'\n')
    textData = '   Primary RA = '+deg_string(ra_p_s) 
    print textData
    txtfile.write(textData+'\n')                  
        
    dec_p_s= deg_decimal_Dec(p_dec_h, p_dec_m, p_dec_s)
    textData = '   Primary Dec = '+degrees_str(p_dec_h, p_dec_m, p_dec_s)   
    print textData
    txtfile.write(textData+'\n')              
    textData = '   Primary Dec = '+deg_string(dec_p_s) 
    print textData
    txtfile.write(textData+'\n')

    ra_s_s = deg_decimal_RA(s_ra_h, s_ra_m, s_ra_s)
    textData = '   Secondary RA = '+hms_str(s_ra_h, s_ra_m, s_ra_s) 
    print textData
    txtfile.write(textData+'\n')              
    textData = '   Secondary RA = '+deg_string(ra_s_s)
    print textData
    txtfile.write(textData+'\n')
         
    dec_s_s = deg_decimal_Dec(s_dec_h, s_dec_m, s_dec_s)
    textData = '   Secondary Dec = '+degrees_str(s_dec_h, s_dec_m, s_dec_s)
    print textData
    txtfile.write(textData+'\n')
    textData = '   Secondary Dec = '+deg_string(dec_s_s)
    print textData
    txtfile.write(textData+'\n')                

    rhos, thetas = rho_theta(ra_p_s, dec_p_s, ra_s_s, dec_s_s)
    textData = ('   Rho = %.3f arcseconds, theta = %.3f degrees' %(rhos, thetas))
    print textData
    txtfile.write(textData+'\n')         

    rcParams['figure.figsize'] = plot_window_x_size, plot_window_y_size
    if auto_aspect_ratio == True:
        axes().set_aspect(str(stretch_RA(dec_p_s)))
    else:
        axes().set_aspect('equal')    
    plt.title('Double Star '+ds_name_str, fontsize=16, fontweight='bold')
    plt.xlabel('RA in Degrees')
    plt.ylabel('Dec in Degrees')
          
    plot_theta(ra_p_s, dec_p_s, rhos, thetas)
    plt.annotate(('Theta = %.3f degrees' %thetas), 
        xy=(ra_p_s, dec_p_s+rhos/(2*3600.)),  
        xycoords='data', color = 'g',
        horizontalalignment='left', verticalalignment='center')     

    plt.plot([ra_p_s,ra_p_s ], [dec_p_s,dec_p_s+rhos/3600.], '--', color = 'g')
    plt.annotate('N', 
        xy=(ra_p_s, dec_p_s+rhos/3600.),  
        xycoords='data', color = 'g',
        horizontalalignment='center', verticalalignment='bottom') 
    plt.plot([ra_p_s,ra_s_s ], [dec_p_s,dec_s_s],  marker='*', markersize=18, color = 'blue')

    image_num_str = str(image_num)
    if ((xlabelp == 0) and (ylabelp == 0)):
        plt.annotate('Primary', 
            xy=(ra_p_s, dec_p_s+rhos*.07*1/3600.),  
            xycoords='data',
            horizontalalignment='center', verticalalignment='center')
    else:
        plt.annotate('Primary', 
            xy=(ra_p_s+offset*xlabelp, dec_p_s+offset*ylabelp),  
            xycoords='data',
            horizontalalignment='center', verticalalignment='center')

    if ((xlabels == 0) and (ylabels == 0)):    
        plt.annotate('Secondary', 
            xy=(ra_s_s, dec_s_s+rhos*.07*1/3600.),
            xycoords='data',
            horizontalalignment='center', verticalalignment='center')  
    else:   
        plt.annotate('Secondary', 
            xy=(ra_s_s+offset*xlabels, dec_s_s+offset*ylabels),
            xycoords='data',
            horizontalalignment='center', verticalalignment='center') 

    plot_rho_circle(rhos, ra_p_s, dec_p_s)
    plot_circles(int(num_circles), circle_size, ra_p_s, dec_p_s)
    if plot_size == 0:
        if auto_aspect_ratio == True:
            plt.xlim(ra_p_s+arcsecond*rhos*1.15*stretch_RA(dec_p_s), ra_p_s-arcsecond*rhos*1.15*stretch_RA(dec_p_s))
            plt.ylim(dec_p_s-arcsecond*rhos*1.15, dec_p_s+arcsecond*rhos*1.15)
            textData = ('\nPlot size is %.2f by %.2f arcseconds.' %(2*rhos*1.15*stretch_RA(dec_p_s),2*rhos*1.15))
            print textData
            txtfile.write(textData+'\n')
            plotPlotSize(2*rhos*1.15*stretch_RA(dec_p_s),2*rhos*1.15)   
        else:
            plt.xlim(ra_p_s+arcsecond*rhos*1.15, ra_p_s-arcsecond*rhos*1.15)
            plt.ylim(dec_p_s-arcsecond*rhos*1.15, dec_p_s+arcsecond*rhos*1.15)
            textData = ('\nPlot size is %.2f by %.2f arcseconds.' %(2*rhos*1.15,2*rhos*1.15))
            print textData
            txtfile.write(textData+'\n')
            plotPlotSize(2*rhos*1.15,2*rhos*1.15)                                 
    else:
        if auto_aspect_ratio == True:
            plt.xlim(ra_p_s+arcsecond*plot_size*.5*stretch_RA(dec_p_s), ra_p_s-arcsecond*plot_size*.5*stretch_RA(dec_p_s))
            plt.ylim(dec_p_s-arcsecond*plot_size*.5, dec_p_s+arcsecond*plot_size*.5) 
            textData = ('\nPlot size is %.2f by %.2f arcseconds.' %(plot_size*stretch_RA(dec_p_s),plot_size))
            print textData
            txtfile.write(textData+'\n')
            plotPlotSize(plot_size*stretch_RA(dec_p_s),plot_size) 
        else:
            plt.xlim(ra_p_s+arcsecond*plot_size*.5, ra_p_s-arcsecond*plot_size*.5)
            plt.ylim(dec_p_s-arcsecond*plot_size*.5, dec_p_s+arcsecond*plot_size*.5) 
            textData = ('\nPlot size is %.2f arcseconds square.' %plot_size)
            print textData
            txtfile.write(textData+'\n')
            plotPlotSize(plot_size,plot_size)                       

    plotImageNum(image_num)
    plotFileNames(python_file_name, data_file_name)
    plotComment(comment)
    plotThetaRho(thetas, rhos)
    plotNSEWlabels()
    plt.grid(True) 
    file_name_date_time = get_date_time()
    savefig(ds_name_str+'_'+file_name_date_time+'.png')                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          

    textData = ('Equal angular adjustment of plot RA is '+str(auto_aspect_ratio)+'.')
    print textData
    txtfile.write(textData+'\n') 

    textData = ('\nEnd of data analysis.')
    print textData
    txtfile.write(textData+'\n') 

    txtfile.close()

def analyze_lines(python_file_name, data_file_name):
    arcsecond = 1/3600.0                         
    offset = 0.1/3600.0

    image_num, p_ra_h, p_ra_m, p_ra_s,  p_dec_h, p_dec_m, p_dec_s, \
    s_ra_h, s_ra_m, s_ra_s, s_dec_h, s_dec_m, \
    s_dec_s, xlabelp, ylabelp, \
    xlabels, ylabels, plot_size, num_circles, circle_size \
    =np.loadtxt(data_file_name, delimiter=',', usecols=(4, 6,7,8, 10,11,12, 14,15,16, 18,19,20, 22,23, 25,26, 28,30,31), unpack=True)

    image_num_str = str(image_num[0])
    rows = p_ra_h.size
    
    thetas = zeros(rows)
    rhos = zeros(rows)
    ra_p_s = zeros(rows)
    dec_p_s = zeros(rows)
    ra_s_s = zeros(rows)
    dec_s_s = zeros(rows)

    data_file = open(data_file_name, 'r')
    first_data_row = data_file.readline()
    data_file.close()

    comma = first_data_row.find(',')
    ds_name_str =  first_data_row[0:comma]
    last_comma = first_data_row.rfind(',')
    comment = first_data_row[last_comma+1:]
    comment = comment.lstrip(' ')
    if ord(comment[len(comment)-1:len(comment)])<32:
        comment = comment = comment[0:len(comment)-1]
    if ord(comment[len(comment)-1:len(comment)])<32:
        comment = comment = comment[0:len(comment)-1]

    file_name_date_time = get_date_time()
    txtfile = open(ds_name_str+'_'+file_name_date_time+'_DataOut.txt', 'w')
    
    now = datetime.datetime.now()
    textData = now.strftime('Date: %Y-%m-%d time: %H:%M') 
    txtfile.write(textData+'\n')
    print textData
    
    textData = 'Python script file name: '+python_file_name+'\n'
    txtfile.write(textData+'\n')
    print textData

    textData = 'Data file: '+data_file_name+' has %.d rows' %rows
    txtfile.write(textData+'\n')
    print textData
    textData = 'Data file contains:\n'
    txtfile.write(textData+'\n')
    print textData        
                        
    data_file = open(data_file_name, 'r')
    for index in range(rows):
        data_row = data_file.readline()
        textData = ('Row %d:' %index)+data_row.rstrip('\n')
        txtfile.write(textData+'\n')
        print textData
    data_file.close()    

    textData = '\nDouble star name:'+ds_name_str     
    print textData
    txtfile.write(textData+'\n')    
    textData = 'Comment: '+comment  
    print textData
    txtfile.write(textData+'\n')


    figure(1)
    rcParams['figure.figsize'] = plot_window_x_size, plot_window_y_size
    plt.title('Double Star '+ds_name_str+' AB', fontsize=16, fontweight='bold')
    plt.xlabel('RA in Degrees')
    plt.ylabel('Dec in Degrees')

    figure(2)
    rcParams['figure.figsize'] = plot_window_x_size, plot_window_y_size
    plt.title('Double Star '+ds_name_str+' C', fontsize=16, fontweight='bold')
    plt.xlabel('RA in Degrees')
    plt.ylabel('Dec in Degrees')

    for row in range(rows):
        textData = ('\nImage %.1f analysis:' %image_num[row])
        print textData
        txtfile.write(textData+'\n')
    
        ra_p_s[row] = deg_decimal_RA(p_ra_h[row], p_ra_m[row], p_ra_s[row])
        textData = '   Primary RA = '+hms_str(p_ra_h[row], p_ra_m[row], p_ra_s[row]) 
        print textData
        txtfile.write(textData+'\n')
        textData = '   Primary RA = '+deg_string(ra_p_s[row]) 
        print textData
        txtfile.write(textData+'\n')                  
        
        dec_p_s[row]= deg_decimal_Dec(p_dec_h[row], p_dec_m[row], p_dec_s[row])
        textData = '   Primary Dec = '+degrees_str(p_dec_h[row], p_dec_m[row], p_dec_s[row])   
        print textData
        txtfile.write(textData+'\n')              
        textData = '   Primary Dec = '+deg_string(dec_p_s[row]) 
        print textData
        txtfile.write(textData+'\n')

        ra_s_s[row] = deg_decimal_RA(s_ra_h[row], s_ra_m[row], s_ra_s[row])
        textData = '   Secondary RA = '+hms_str(s_ra_h[row], s_ra_m[row], s_ra_s[row]) 
        print textData
        txtfile.write(textData+'\n')              
        textData = '   Secondary RA = '+deg_string(ra_s_s[row])
        print textData
        txtfile.write(textData+'\n')
         
        dec_s_s[row] = deg_decimal_Dec(s_dec_h[row], s_dec_m[row], s_dec_s[row])
        textData = '   Secondary Dec = '+degrees_str(s_dec_h[row], s_dec_m[row], s_dec_s[row])
        print textData
        txtfile.write(textData+'\n')
        textData = '   Secondary Dec = '+deg_string(dec_s_s[row])
        print textData
        txtfile.write(textData+'\n')                

        rhos[row], thetas[row] = rho_theta(ra_p_s[row], dec_p_s[row], ra_s_s[row], dec_s_s[row])
        textData = ('   Rho = %.5f arcseconds, theta = %.5f degrees' %(rhos[row], thetas[row]))
        print textData
        txtfile.write(textData+'\n')         
        
        image_num_str = str(image_num[row])
        
        figure(1)
        plt.plot([ra_p_s[row],ra_s_s[row] ], [dec_p_s[row],dec_s_s[row]],  marker='*', markersize=18)
        if (xlabelp[row]!= 0)or(ylabelp[row] != 0):
            plt.annotate(image_num_str, 
                    xy=(ra_p_s[row]+offset*xlabelp[row], dec_p_s[row]+offset*ylabelp[row]),  
                    xycoords='data',
                    horizontalalignment='center', verticalalignment='center')
        if (xlabels[row]!= 0)or(ylabels[row] != 0):
            plt.annotate(image_num_str, 
                    xy=(ra_s_s[row]+offset*xlabels[row], dec_s_s[row]+offset*ylabels[row]),
                    xycoords='data',
                    horizontalalignment='center', verticalalignment='center') 

        figure(2)
        plt.plot([ra_p_s[row],ra_s_s[row] ], [dec_p_s[row],dec_s_s[row]],  marker='*', markersize=18)
        if (xlabelp[row]!= 0)or(ylabelp[row] != 0):
            plt.annotate(image_num_str, 
                    xy=(ra_p_s[row]+offset*xlabelp[row], dec_p_s[row]+offset*ylabelp[row]),  
                    xycoords='data',
                    horizontalalignment='center', verticalalignment='center')
        if (xlabels[row]!= 0)or(ylabels[row] != 0):
            plt.annotate(image_num_str, 
                    xy=(ra_s_s[row]+offset*xlabels[row], dec_s_s[row]+offset*ylabels[row]),
                    xycoords='data',
                    horizontalalignment='center', verticalalignment='center') 

    textData = ('\n%d Rho & theta measurements analysis:' %rows)
    print textData
    txtfile.write(textData+'\n') 
        
    for row in range(rows):
        textData = ('   Image %.1d, rho = %.5f arcseconds, theta = %.5f degrees' %(image_num[row], rhos[row], thetas[row]) )
        print textData
        txtfile.write(textData+'\n') 

    textData = ('\n   Rho mean = %.5f arcseconds' %(rhos.mean()))
    print textData
    txtfile.write(textData+'\n')

    textData = ('   Rho standard deviation = %.5f arcseconds' %(rhos.std()))
    print textData
    txtfile.write(textData+'\n')

    textData = ('   Rho standard error of the mean = %.5f arcseconds' %(rhos.std()/sqrt(rows)))
    print textData
    txtfile.write(textData+'\n')
     
    textData = ('   Rho mean - standard deviation = %.5f arcseconds' %(rhos.mean()-rhos.std()))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('   Rho mean + standard deviation = %.5f arcseconds' %(rhos.mean()+rhos.std()))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('   Maximum rho = %.5f arcseconds in image %.1f' %(rhos.max(), image_num[rhos.argmax()]))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('   Minimum rho = %.5f arcseconds in image %.1f' %(rhos.min(), image_num[rhos.argmin()]))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('   Rho spread = %.5f arcseconds, rho spread percentage of rho mean %.2f%%' %((rhos.max()-rhos.min()),100*(rhos.max()-rhos.min())/rhos.mean()))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('\n   Theta mean = %.5f degrees' %(thetas.mean()))
    print textData
    txtfile.write(textData+'\n')
 
    textData = ('   Theta standard deviation = %.5f degrees' %(thetas.std()))
    print textData
    txtfile.write(textData+'\n')

    textData = ('   Theta standard error of the mean = %.5f arcseconds' %(thetas.std()/sqrt(rows)))
    print textData
    txtfile.write(textData+'\n')
                               
    textData = ('   Theta mean - standard deviation = %.5f degrees' %(thetas.mean()-thetas.std()))
    print textData
    txtfile.write(textData+'\n')
          
    textData = ('   Theta mean + standard deviation = %.5f degrees' %(thetas.mean()+thetas.std()))
    print textData
    txtfile.write(textData+'\n')
 
    textData = ('   Maximum theta  = %.5f degrees in image %.1f' %(thetas.max(), image_num[thetas.argmax()]))
    print textData
    txtfile.write(textData+'\n')
          
    textData = ('   Minimum theta  = %.5f degrees in image %.1f' %(thetas.min(), image_num[thetas.argmin()]))
    print textData
    txtfile.write(textData+'\n')
          
    textData = ('   Theta spread = %.5f degrees' %(thetas.max()-thetas.min()))
    print textData
    txtfile.write(textData+'\n')
 
    textData = ('   Theta spread  = %.5f arcminutes' %(60*(thetas.max()-thetas.min())))
    print textData
    txtfile.write(textData+'\n')

    textData = ('   Theta spread  = %.5f arcseconds' %(3600*(thetas.max()-thetas.min())))
    print textData
    txtfile.write(textData+'\n')
                                                                        
    textData = ('\nMean of %d images RA & Dec analysis:' %rows)
    print textData
    txtfile.write(textData+'\n')

    textData = ('   Mean of primary RA & Dec:')
    print textData
    txtfile.write(textData+'\n')

    textData = ('      RA mean = %.5f degrees' %(ra_p_s.mean()))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      RA standard deviation = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(ra_p_s.std(),ra_p_s.std()*60.,ra_p_s.std()*3600.))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      RA standard error of the mean = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(ra_p_s.std()/sqrt(rows),ra_p_s.std()*60./sqrt(rows),ra_p_s.std()*3600./sqrt(rows)))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('      Dec = %.5f degrees' %(dec_p_s.mean()))
    print textData
    txtfile.write(textData+'\n')
 
    textData = ('      Dec standard deviation = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(dec_p_s.std(),dec_p_s.std()*60.,dec_p_s.std()*3600.))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      Dec standard error of the mean = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(dec_p_s.std()/sqrt(rows),dec_p_s.std()*60./sqrt(rows),dec_p_s.std()*3600./sqrt(rows)))
    print textData
    txtfile.write(textData+'\n')
    
    textData = ('   Mean of secondary RA & Dec:')
    print textData
    txtfile.write(textData+'\n')

    textData = ('      RA = %.5f degrees' %(ra_s_s.mean()))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      RA standard deviation = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(ra_s_s.std(),ra_s_s.std()*60.,ra_s_s.std()*3600.))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      RA standard error of the mean = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(ra_s_s.std()/sqrt(rows),ra_s_s.std()*60./sqrt(rows),ra_s_s.std()*3600./sqrt(rows)))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      Dec = %.5f degrees' %(dec_s_s.mean()))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      Dec standard deviation = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(dec_s_s.std(),dec_p_s.std()*60.,dec_p_s.std()*3600.))
    print textData
    txtfile.write(textData+'\n')

    textData = ('      Dec standard error of the mean = %.7f degrees, %.7f arcminutes, %.7f arcseconds' %(dec_s_s.std()/sqrt(rows),dec_s_s.std()*60./sqrt(rows),dec_s_s.std()*3600./sqrt(rows)))
    print textData
    txtfile.write(textData+'\n')
    
    rho_mean, theta_mean = rho_theta(ra_p_s.mean(), dec_p_s.mean(), ra_s_s.mean(), dec_s_s.mean())

    textData = ('   Rho = %.5f arcseconds, theta = %.5f degrees' %(rho_mean, theta_mean))
    print textData
    txtfile.write(textData+'\n')      

    textData = ('Plot %d circles in radii increments of %.2f arcseconds.' %(num_circles[row], circle_size[row]))
    print textData
    txtfile.write(textData+'\n')    

    file_name_date_time = get_date_time()
    plot_double_star(python_file_name, data_file_name, txtfile, ds_name_str, comment, ra_p_s.mean(), dec_p_s.mean(), ra_s_s.mean(), dec_s_s.mean(), rho_mean, theta_mean)
    
    figure(1)
    if auto_aspect_ratio == True:
        axes().set_aspect(str(stretch_RA(dec_p_s.mean())))
    else:
        axes().set_aspect('equal') 
    plt.plot([ra_p_s.mean(),ra_s_s.mean()], [dec_p_s.mean(), dec_s_s.mean()], color='black', marker='+', markersize=18)
    plot_circles(int(num_circles[row]), circle_size[row], ra_p_s.mean(), dec_p_s.mean())
    plot_circles(int(num_circles[row]), circle_size[row], ra_s_s.mean(), dec_s_s.mean())           
    if auto_aspect_ratio == True:
        plt.xlim(ra_p_s.mean()+arcsecond*plot_size[row]*.5*stretch_RA(dec_p_s.mean()), ra_p_s.mean()-arcsecond*plot_size[row]*.5*stretch_RA(dec_p_s.mean()))
        plt.ylim(dec_p_s.mean()-arcsecond*plot_size[row]*.5, dec_p_s.mean()+arcsecond*plot_size[row]*.5) 
        textData = ('Primary plot size is %.2f by %.2f arcseconds.' %(plot_size[row]*stretch_RA(dec_p_s.mean()),plot_size[row]))
        print textData
        txtfile.write(textData+'\n')
        plotPlotSize(plot_size[row]*stretch_RA(dec_p_s.mean()),plot_size[row]) 
    else:
        plt.xlim(ra_p_s.mean()+arcsecond*plot_size[row]*.5, ra_p_s.mean()-arcsecond*plot_size[row]*.5)
        plt.ylim(dec_p_s.mean()-arcsecond*plot_size[row]*.5, dec_p_s.mean()+arcsecond*plot_size[row]*.5)
        textData = ('Primary plot size is %.2f arcseconds square.' %plot_size)
        print textData
        txtfile.write(textData+'\n')
        plotPlotSize(plot_size,plot_size)   
    plotFileNames(python_file_name, data_file_name)
    plotComment(comment)
    plotAverageLabels(ra_p_s.mean(), dec_p_s.mean(), ra_s_s.mean(), dec_s_s.mean())
    plotNSEWlabels()
    plt.grid(True)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
    savefig(ds_name_str+'_AB_'+file_name_date_time+'.png')
    
    figure(2)
    if auto_aspect_ratio == True:
        axes().set_aspect(str(stretch_RA(dec_s_s.mean())))
    else:
        axes().set_aspect('equal') 
    plt.plot([ra_p_s.mean(),ra_s_s.mean()], [dec_p_s.mean(), dec_s_s.mean()], color='black', marker='+', markersize=18)
    plot_circles(int(num_circles[row]), circle_size[row], ra_p_s.mean(), dec_p_s.mean())
    plot_circles(int(num_circles[row]), circle_size[row], ra_s_s.mean(), dec_s_s.mean())
    if auto_aspect_ratio == True:
        plt.xlim(ra_s_s.mean()+arcsecond*plot_size[row]*.5*stretch_RA(dec_s_s.mean()), ra_s_s.mean()-arcsecond*plot_size[row]*.5*stretch_RA(dec_s_s.mean()))
        plt.ylim(dec_s_s.mean()-arcsecond*plot_size[row]*.5, dec_s_s.mean()+arcsecond*plot_size[row]*.5) 
        textData = ('Secondary plot size is %.2f by %.2f arcseconds.' %(plot_size[row]*stretch_RA(dec_s_s.mean()),plot_size[row]))
        print textData
        txtfile.write(textData+'\n')
        plotPlotSize(plot_size[row]*stretch_RA(dec_s_s.mean()),plot_size[row]) 
    else:
        plt.xlim(ra_s_s.mean()+arcsecond*plot_size[row]*.5, ra_s_s.mean()-arcsecond*plot_size[row]*.5)
        plt.ylim(dec_s_s.mean()-arcsecond*plot_size[row]*.5, dec_s_s.mean()+arcsecond*plot_size[row]*.5)
        textData = ('\Secondary plot size is %.2f arcseconds square.' %plot_size[row])
        print textData
        txtfile.write(textData+'\n')
        plotPlotSize(plot_size,plot_size) 
    plotFileNames(python_file_name, data_file_name)
    plotComment(comment)
    plotAverageLabels(ra_p_s.mean(), dec_p_s.mean(), ra_s_s.mean(), dec_s_s.mean())
    plotNSEWlabels()
    plt.grid(True)
    savefig(ds_name_str+'_C_'+file_name_date_time+'.png')                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 

    textData = ('Equal angular adjustment of plot RA is '+str(auto_aspect_ratio)+'.')
    print textData
    txtfile.write(textData+'\n') 
    
    textData = ('\nEnd of data analysis.')
    print textData
    txtfile.write(textData+'\n') 

    txtfile.close()
    
def main():
    python_file = get_script_name()
    data_file = get_data_file_name()
    row_count = get_row_count(data_file)
    if row_count > 1:
        analyze_lines(python_file, data_file)
    else:
        analyze_one_line(python_file, data_file)
    plt.show()
    
main()